#!/bin/sh
# $Id
#
# Remark: 07/10/08: On request from Sandra, flag 
#         CFLAGS="-D_LARGEFILE_SOURCE=1 -D_FILE_OFFSET_BITS=64" 
#         where added to the compilation of cfitsio and cpl. They will not 
#         be necessary with the next release of cfitsio 3 and cpl 5.0 as they
#         will be included by default. 

install_cfitsio()
{
cd $THISPWD/tmp
if [ -f ../${CFITSIO_PKG}.tar.gz ];then 
  if [ -d ${CFITSIO} ]; then
    rm -rf ${CFITSIO}.old
    mv ${CFITSIO}  ${CFITSIO}.old
  fi
  gunzip -c ../${CFITSIO_PKG}.tar.gz | tar xf -
  cd ${CFITSIO}
  if ! ./configure --prefix=$DESTPWD ; then
    return 1
  fi
  # This line added on request from Sandra for VISTA/OmegaCAM pipelines
  if ! make CFLAGS="-D_LARGEFILE_SOURCE=1 -D_FILE_OFFSET_BITS=64" ; then
    return 1
  fi
  if ! make shared ; then
    return 1
  fi
  if ! make install ; then
    return 1
  fi
else
  echo "Error: Package ${CFITSIO}.tar.gz not found"
  exit 1
fi
CFITSIODIR=$DESTPWD; export CFITSIODIR
echo "CFITSIO installation finished"
return 0
}

install_wcslib()
{
#
# CG: WCSLIB requires the directory $DESTPWD/bin to exist, therefore
#     I create all of them: bin, lib and include
mkdir -p $DESTPWD/bin $DESTPWD/lib $DESTPWD/include

cd $THISPWD/tmp
if [ -f ../${WCSLIB}.tar.gz ];then 
  if [ -d ${WCSLIB} ]; then
    rm -rf ${WCSLIB}.old
    mv ${WCSLIB}  ${WCSLIB}.old
  fi
  gunzip -c ../${WCSLIB}.tar.gz | tar xf -
  cd ${WCSLIB}
  if ! ./configure --prefix=$DESTPWD ; then
    return 1
  fi
  if ! make all; then
    return 1
  fi
  if ! make install; then
    return 1
  fi
  cp wcsconfig.h $DESTPWD/include/${WCSLIB}
else
  echo "Error: Package ${WCSLIB}.tar.gz not found"
  return 1
fi
WCSDIR=$DESTPWD; export WCSDIR
echo "WCS installation finished"
return 0
}

install_cpl()
{
cd $THISPWD/tmp
if [ -f ../${CPL}.tar.gz ];then 
  if [ -d ${CPL} ]; then
    rm -rf ${CPL}.old
    mv  ${CPL} ${CPL}.old
  fi
  gunzip -c ../${CPL}.tar.gz  | tar xf -
  if [ -z "$CFITSIODIR" -o ! -d "$CFITSIODIR" ]; then
    echo "Error: CFITSIODIR not defined or directory does not exist"
    return 1
  fi
  if [ -z "$WCSDIR" -o ! -d "$WCSDIR" ]; then
    echo "Error: WCSDIR not defined or directory does not exist"
    return 1
  fi
  cd ${CPL}
  CXXFLAGS="-g -O0"; export CXXFLAGS
  CFLAGS="-g -O0 -fno-builtin -fno-common"; export CFLAGS
  if ! ./configure --prefix=$DESTPWD ; then
    return 1
  fi
  if ! make install ; then
    return 1
  fi
  unset CXXFLAGS
  unset CFLAGS
else
  echo "Error: Package ${CPL}.tar.gz not found"
  return 1
fi
CPLDIR=$DESTPWD; export CPLDIR
echo "CPL installation finished"
return 0
}

install_esorex()
{
cd $THISPWD/tmp
if [ -f ../${ESOREX}.tar.gz ];then 
  if [ -d ${ESOREX} ]; then
    rm -rf ${ESOREX}.old
    mv  ${ESOREX} ${ESOREX}.old
  fi
  gunzip -c ../${ESOREX}.tar.gz  | tar xf -
  if [ ! -d ${ESOREX} ]; then 
    echo "Error: directory ${ESOREX} not found"
    return 1
  fi
  if [ -z "$CPLDIR" -o ! -d "$CPLDIR" ]; then
    echo "Error: CPLDIR not defined or directory does not exist"
    return 1
  fi
  if [ -z "$WCSDIR" -o ! -d "$WCSDIR" ]; then
    echo "Error: WCSDIR not defined or directory does not exist"
    return 1
  fi
  cd ${ESOREX}
  if ! ./configure --prefix=$DESTPWD --with-cpl=$CPLDIR --with-cfitsio=$CFITSIOSDIR ; then 
    return 1
  fi
  if ! make  ; then
    return 1
  fi
  if ! make install; then
    return 1
  fi
else
  echo "Error: Package ${ESOREX}.tar.gz not found"
  return 1
fi
echo "ESOREX installation finished"
return 0
}

# Main, main, MAIN

CFITSIO=cfitsio
CFITSIO_PKG=cfitsio2510
CPL=cpl-4.1.1 
ESOREX=esorex-3.6.8
DFS=CPL
WCSLIB=wcslib-4.3

THISPWD=`dirname $0`
THISPWD=`(cd $THISPWD ; /bin/pwd)`
if [ -z $INTROOT ]; then 
  echo "Error: INTROOT not defined"
  exit 1
elif [ ! -d $INTROOT ]; then
  echo "Error: INTROOT=$INTROOT not such directory"
  exit 1
fi

if [ -d $INTROOT/gasgano ]; then
  GASGANODIR=$INTROOT/gasgano ; export GASGANODIR
fi

DESTPWD=${INTROOT}/cpl

cd $THISPWD
rm -rf $DESTPWD tmp
mkdir -p $DESTPWD tmp

cmd=`basename $0`

# Using the one from VLT2009
#if ! install_cfitsio ; then
#  echo "Error installing CFITSIO. Exit"
#  exit 1
#fi

if ! install_wcslib ; then
  echo "Error installing WCSLIB. Exit"
  exit 1
fi

if ! install_cpl ; then
  echo "Error installing CPL. Exit"
  exit 1
fi

#if ! install_esorex ; then
#  echo "Error installing ESOREX. Exit"
#  exit 1
#fi

exit 0
